# -*- coding: utf-8 -*-
"""
Created on Mon Jun 17 13:24:28 2024

@author: nash29
"""

import numpy as np
from yamdb import yamdb as ydb
import matplotlib.pyplot as plt
from labellines import labelLine, labelLines

# Inter-axis conversions
def i_to_frac(i_sd):
    return i_sd/1.89
def frac_to_i(frac):
    return 1.89*frac

fig = plt.figure(figsize=(7,7))
ax = plt.subplot(111)
ax_twin = ax.secondary_yaxis('right', functions=(i_to_frac, frac_to_i))

#ax_twin.set_yticks(1)
ax_twin.set_ylabel('Fraction of total cell capacity lost per hour (-)')

# Define diffusion coefficient range
D_range = np.logspace(-3.75,-6.75,100)

# Theoretical diaphragms
effective_area_range = [0.2, 0.4, 0.6, 0.8, 1.0]
labels = []
for ea in effective_area_range:
    labels.append(ea)

for h, colour in zip([5.0,0.5],['silver','k']):
    for effective_area, label in zip(effective_area_range, labels):
        #D_range = np.logspace(-4.5,-7.5,100)
        conc_catholyte = 0.004               # mol/cm^3
        #h = 0.5                                # diaphragm thickness
        gradient = conc_catholyte/h             # mol/cm^4
        #effective_area = 0.1                   # -
        
        J = D_range*gradient*effective_area
        
        i_self_discharge = J*96485*2
        
        capacity = 1
        fractions = i_self_discharge/capacity
        
        ax.plot(D_range,i_self_discharge, label=label, color=colour) 

# Real experimental values
conc_catholyte = 0.004                  # mol/cm^3
#h = 0.5                                # diaphragm thickness
gradient = conc_catholyte/0.5             # mol/cm^4
#effective_area = 0.1                   # -

J = D_range*gradient*0.9

i_self_discharge = J*96485*2

capacity = 1
fractions = i_self_discharge/capacity
ax.plot(D_range,i_self_discharge, label=0.9, color='blue')

# Plot settings
x_values = np.logspace(-3.75,-7,15)[1:]
labelLines(ax.get_lines(), xvals = x_values, zorder=2)

ax.set_xlabel(r'Zn$^{2+}$ Diffusion coefficient (cm$^2$/s)')
ax.set_ylabel('Maximum self-discharge current (A)')
ax.set_xscale('log')
ax.set_yscale('log')
#ax.grid(linestyle='--')
ax.legend(['Thickness: 5.0 cm', '_', '_', '_', '_', 'Thickness: 0.5 cm', '_', '_', '_', '_', 'Experimental diaphragm'], loc='lower right')
ax.set_xlim(min(D_range),max(D_range))
ax.set_ylim(bottom = 0.0002, top = 0.25)
ax.axvline(x=8.12E-6, ymin=1E-8, ymax=1, linestyle='--', color='lightgrey', zorder=-1)
ax.annotate('600$\degree$C', xy=(1.0E-5,0.35), color='grey')

# Inset plot
R = 1.987
T_range = np.linspace(803,898,100)
D = 0.0585*np.exp((-15400)/(R*T_range))
T_range_exp = np.linspace(673,803,100)
D_exp = 0.0585*np.exp((-15400)/(R*T_range_exp))

left, bottom, width, height = [0.24, 0.62, 0.22, 0.22]
ax2 = fig.add_axes([left, bottom, width, height])
ax2.plot(T_range-273,D, color = 'k', linestyle = '--')
ax2.plot(T_range_exp-273,D_exp, color = 'k')
ax2.set_xlabel('Temperature ($^\circ$C)')
ax2.set_ylabel(r'D$_{Zn^{2+}}$ (cm$^2$/s)')
